//--------------------------------------------------------------------
//
//  Modul test_ram - Version mit variabel programmierbarer Aufloesung
//                   und beschreibbarem Videospeicher
//
//  Testumgebung fuer den Display-Controller
//
//--------------------------------------------------------------------

`include "discount_defs.v"

module test_ram;

  `define Simtime ((`Hline)*(`Vframe)*120)  // 1.2 Frames simulieren
  
  // Deklarationen
  reg             CLOCK,          // Takt
                  NRST,           // Initialisierung
                  WE,             // Write-Enable Programmierbus
                  SNAPSHOT;       // Bild speichern (PBM)
  reg  [`Asz-1:0] PADDR;          // Programmier-Adressbus
  wire [`Rsz-1:0] PDATA;          // Bidirektionaler Programmierdatenbus
  reg  [`Rsz-1:0] PDATA_REG;      // Bidirektionaler Programmierdatenbus
  wire            PIXEL,          // Bildpunkt
                  CSYNC,          // kombinierte Synchronisation
                  HSYNC,          // horizontale Synchronisation
                  VSYNC;          // vertikale Synchronisation
  integer         H, V;           // Bildposition
  reg  [0:34655]  BITMAP[0:0];    // Zwischenspeicher fuer Test-Bitmap

  //
  // Testinstanz des Display-Controllers
  //
  discount_ram DISCOUNT (PIXEL,CSYNC,HSYNC,VSYNC,
                         CLOCK,NRST,PADDR,WE,PDATA);

  //
  // Instanz des Testmonitors (schreibt PBM)
  //
  crt CRT (PIXEL,CSYNC,HSYNC,VSYNC,SNAPSHOT,CLOCK);

  //
  // Bidirektionalen Bus treiben
  assign PDATA = PDATA_REG;

  //
  // Takt erzeugen
  //
  always begin
    CLOCK = 1'b0;
    #50;
    CLOCK = 1'b1;
    #50;
  end

  //
  // Reset
  //
  initial begin
    NRST = 1'b0;
    @(negedge CLOCK);
    NRST = 1'b1;
  end

  initial
    #`Simtime $stop;                    // Simulationsdauer

  //
  // Textausgabe
  //
  initial
    $monitor("Takt=%d  PIXEL=%b  HSYNC=%b  VSYNC=%b",
      $time/100, PIXEL, HSYNC, VSYNC);

  //
  // Snapshot speichern
  //
  initial
    SNAPSHOT = 1;

  //
  // Aufloesung programmieren
  // Test-Bitmap via Programmierbus in den Bildspeicher schreiben
  //
  initial begin
    PADDR = 0; WE = 0; PDATA_REG = `Rsz'bz;
    #200;
    @(negedge CLOCK);
	 // Aufloesung 320*200 Punkte
    WE = 1; PDATA_REG = 48; #100;
    PADDR = 1; PDATA_REG = 320; #100;
    PADDR = 2; PDATA_REG = 44; #100;
    PADDR = 3; #100;
    PADDR = 4; PDATA_REG = 60; #100;
    PADDR = 5; PDATA_REG = 200; #100;
    PADDR = 6; PDATA_REG = 50; #100;
    PADDR = 7; PDATA_REG = 16; #100;
    // Wieder auslesen
    WE = 0; PDATA_REG = `Rsz'bz; PADDR = 1; #100;
    $display("Neue Aufloesung: x %d", PDATA);
    PADDR = 5; #100;
    $display("Neue Aufloesung: y %d", PDATA);
    //
    // Test-Bitmap aus Datei in temporre Variable lesen
    $readmemb("athene_logo_sw_304x114.mem", BITMAP);
    // Bitmap via Programmierbus zeilenweise in Bildspeicher
    // schreiben, so dass die Bitmap zentriert erscheint
    // Bitmap hat 304x114 Punkte => V: 42-156, H: 8-312
	 $display("Schreibe Bitmap in Videospeicher...");
    PADDR = 1 + (1 << 14); // Videospeicher adressieren
    WE = 1;                // schreiben
    for (V = 0; V < 114; V = V + 1) begin
      for (H = 0; H < 304; H = H + `Bsz) begin
        // Jeweils 8 Bit via Programmierbus schreiben,
        // einen Takt warten
        PDATA_REG = BITMAP[0][V*304+H +: 8]; #100;
        // Adresse hochzaehlen
        PADDR = PADDR + 1;
      end
      // Am Zeilenende 16 Pixel ueberspringen
      // (Breite der Bitmap nur 304)
      PADDR = PADDR + 2;
    end
    WE = 0; PDATA_REG = `Rsz'bz;
	 $display("Bitmap schreiben beendet.");
  end

endmodule // test_ram
